import argparse
import itertools
import pathlib

import arcticdb
import pandas as pd
import yaml
from trading_ig import IGService
from trading_ig.config import config

S3_BUCKET_NAME = "tradingo-price-store"
ARCTIC_CONNECTION = (
    f"s3://s3.eu-west-2.amazonaws.com:{S3_BUCKET_NAME}?aws_auth=true"
)
ARCTIC_CONNECTION = "lmdb:///home/rory/.trading/.tradingo-db.test"


def extract(
    arctic,
    epics,
    start_date: pd.Timestamp,
    end_date: pd.Timestamp,
    resolution: str = "1D",
):
    ig_service = IGService(
        config.username, config.password, config.api_key, config.acc_type
    )
    ig_service.create_session()

    lib = arctic.get_library(
        f"INSTRUMENTS_{resolution.upper()}",
        create_if_missing=True,
        library_options=arcticdb.LibraryOptions(dedup=True),
    )

    for epic in epics:
        meta = ig_service.fetch_market_by_epic(epic)

        data = ig_service.fetch_historical_prices_by_epic_and_date_range(
            epic=epic,
            resolution=resolution,
            end_date=end_date,
            start_date=start_date,
        )

        prices = data["prices"]

        prices.columns = [
            "_".join(i).lower() for i in prices.columns.to_series()
        ]

        lib.update(
            epic,
            prices,
            upsert=True,
            date_range=(start_date, end_date),
            metadata=dict(meta),
        )


def transform(
    arctic: arcticdb.Arctic,
    epics,
    universe_name: str,
    start_date: pd.Timestamp,
    end_date: pd.Timestamp,
    resolution: str = "1D",
):
    in_lib = arctic.get_library(f"INSTRUMENTS_{resolution}")
    out_lib = arctic.get_library(
        f"{universe_name.upper()}_PRICE_{resolution}",
        create_if_missing=True,
        library_options=arcticdb.LibraryOptions(dedup=True),
    )

    for field, obvs in itertools.product(
        ("bid", "ask"), ("open", "close", "high", "low")
    ):
        symbol = f"{field}.{obvs}".upper()

        out_df = pd.concat(
            (
                in_lib.read(
                    symbol,
                    columns=[f"{field}_{obvs}"],
                    date_range=(start_date, end_date),
                )
                .data.squeeze(axis="columns")
                .rename(symbol)
                for symbol in epics
            ),
            axis=1,
        )

        metadata = {"universe": epics, "universe_name": universe_name}

        out_lib.update(
            symbol,
            out_df,
            upsert=True,
            date_range=(start_date, end_date),
            metadata=metadata,
        )


def load(
    arctic: arcticdb.Arctic,
    epics,
    universe_name: str,
    start_date: pd.Timestamp,
    end_date: pd.Timestamp,
    resolution: str = "1D",
):
    lib = arctic.get_library(f"{universe_name.upper()}_PRICE_{resolution}")

    data = {
        fld: lib.read(
            "_".join(fld).upper(),
            date_range=(start_date, end_date),
            columns=epics,
        ).data
        for fld in itertools.product(
            ("bid", "ask"), ("open", "close", "high", "low")
        )
    }
    return pd.concat(data.values(), keys=data.keys(), axis=1)


def cli_app():
    app = argparse.ArgumentParser("tradingo-price-etl")
    app.add_argument("actions", nargs="+")
    app.add_argument("--epics", required=False, nargs="+")
    app.add_argument("--universe-file", required=False, type=pathlib.Path)
    app.add_argument("--resolution", default="1D")
    app.add_argument(
        "--end-date", type=pd.Timestamp, default=pd.Timestamp.now()
    )
    app.add_argument("--start-date", type=pd.Timestamp)
    app.add_argument("--universe-name", required=True)

    return app


def main():
    args = cli_app().parse_args()
    universe_config = yaml.load(args.universe_file.open("r"), yaml.Loader)

    if "epics" in universe_config:
        args.epics = universe_config["epics"]

    if "resolution" in universe_config:
        args.resolution = universe_config["resolution"]

    if "universe_name" in universe_config:
        args.universe_name = universe_config["universe_name"]

    arctic = arcticdb.Arctic(ARCTIC_CONNECTION)

    for action in args.actions:
        if action == "extract":
            extract(
                arctic,
                epics=args.epics,
                start_date=args.start_date,
                end_date=args.end_date,
                resolution=args.resolution,
            )

        elif action == "transform":
            transform(
                arctic,
                epics=args.epics,
                start_date=args.start_date,
                universe_name=args.universe_name,
                end_date=args.end_date,
                resolution=args.resolution,
            )

        elif action == "load":
            load(
                arctic,
                universe_name=args.universe_name,
                epics=args.epics,
                start_date=args.start_date,
                end_date=args.end_date,
                resolution=args.resolution,
            )

        elif action == 'run':
            raise NotImplementedError

        else:
            raise ValueError(f"Invalid action {action}")


if __name__ == "__main__":
    main()



from trading_ig import IGService
from trading_ig.config import config

ig_service = IGService(
    config.username, config.password, config.api_key, config.acc_type
)
ig_service.create_session()

open_positions = ig_service.fetch_open_positions()
print("open_positions:\n%s" % open_positions)

print("")


epic = "KB.D.DRINKS.CASH.IP"  # 'CS.D.EURGBP.MINI.IP'
resolution = "1min"
num_points = 10
response = ig_service.fetch_historical_prices_by_epic_and_num_points(
    epic, resolution, num_points
)
df_ask = response["prices"]["bid"]
print("ask prices:\n%s" % df_ask)



from zipline.api import order_target, record, symbol


def initialize(context):
    context.i = 0
    context.asset = symbol("AAPL")


def handle_data(context, data):
    # Skip first 300 days to get full windows
    context.i += 1
    if context.i < 300:
        return

    # Compute averages
    # data.history() has to be called with the same params
    # from above and returns a pandas dataframe.
    short_mavg = data.history(
        context.asset, "price", bar_count=100, frequency="1d"
    ).mean()
    long_mavg = data.history(
        context.asset, "price", bar_count=300, frequency="1d"
    ).mean()

    # Trading logic
    if short_mavg > long_mavg:
        # order_target orders as many shares as needed to
        # achieve the desired number of shares.
        order_target(context.asset, 100)
    elif short_mavg < long_mavg:
        order_target(context.asset, 0)

    # Save values for later inspection
    record(
        AAPL=data.current(context.asset, "price"),
        short_mavg=short_mavg,
        long_mavg=long_mavg,
    )



import numpy as np
import pandas as pd


def log_returns(dollar_prices):
    return np.log(dollar_prices / dollar_prices.shift()).dropna()


def calculate_trades(
    position: pd.DataFrame,
) -> pd.DataFrame:
    trades = position.diff()
    return trades[trades != 0].dropna(axis=1, how="all").fillna(0)


def trade_balance(
    trades: pd.DataFrame,
    dollar_prices: pd.DataFrame,
) -> pd.DataFrame:
    return (trades * dollar_prices).cumsum()


def realised_pnl(
    positions: pd.DataFrame,
    dollar_prices: pd.DataFrame,
) -> pd.DataFrame:
    trades = calculate_trades(positions)
    reindexed_positions = positions.reindex_like(trades)
    reindexed_prices = dollar_prices.reindex_like(trades)
    return (-1 * (trades * reindexed_prices)).cumsum() + (
        reindexed_positions * reindexed_prices
    )


def unrealised_pnl(
    positions: pd.DataFrame,
    dollar_prices: pd.DataFrame,
) -> pd.DataFrame:
    return (positions.multiply(dollar_prices)).diff().fillna(0)


def total_pnl(
    trades: pd.DataFrame,
    positions: pd.DataFrame,
    dollar_prices: pd.DataFrame,
) -> pd.DataFrame:
    return unrealised_pnl(trades, positions, dollar_prices)


def sharpe_ratio(
    positions: pd.DataFrame,
    dollar_prices: pd.DataFrame,
):
    trades = calculate_trades(positions)
    pnl = total_pnl(
        trades=trades, positions=positions, dollar_prices=dollar_prices
    ).sum(axis=1)
    ewm = log_returns(pnl).ewm(halflife=120)
    return ewm.mean() / ewm.std()



import time

import pandas as pd
import requests
from trading_ig.config import config

IG_BASE_URL = "https://demo-api.ig.com/gateway/deal"


class IG:
    def __init__(
        self,
        username,
        password,
        api_key,
        base_url="https://api.ig.com/gateway/deal",
    ):
        self.session_init(username, password, api_key, base_url)
        self.markets = []

    def session_init(self, username, password, api_key, base_url):
        self.base_url = base_url
        self.headers = {
            "Content-Type": "application/json; charset=UTF-8",
            "Accept": "application/json; charset=UTF-8",
            "VERSION": "2",
        }
        self.headers["X-IG-API-KEY"] = api_key
        self.session = requests.post(
            f"{self.base_url}/session",
            json={
                "identifier": username,
                "password": password,
                "encryptedPassword": False,
            },
            headers=self.headers,
        )
        self.headers["CST"] = self.session.headers["CST"]
        self.headers["X-SECURITY-TOKEN"] = self.session.headers[
            "X-SECURITY-TOKEN"
        ]
        self.headers_v1 = dict(self.headers)
        self.headers_v1["VERSION"] = "1"

    def navigation(self, node_id):
        return requests.get(
            f"{self.base_url}/marketnavigation/{node_id}",
            headers=self.headers_v1,
        ).json()

    def traverse(self, node_id, delay=2):
        level = self.navigation(node_id)
        time.sleep(delay)
        if "nodes" in level and isinstance(level["nodes"], list):
            for node in level["nodes"]:
                self.traverse(node["id"])
        if "markets" in level and isinstance(level["markets"], list):
            for market in level["markets"]:
                self.markets.append(market)

        return self.markets

    def get_instruments(self, epics=(), filt="ALL"):
        return requests.get(
            f"{self.base_url}/markets",
            params={"epics": ",".join(epics), "filter": filt},
            headers=self.headers,
        ).json()


config.get


ig = IG(config.username, config.password, config.api_key, base_url=IG_BASE_URL)

markets = ig.traverse(".")

data = pd.DataFrame(markets)

data.to_csv("./ig-markets.csv", index=False)
